package danet

import (
	"fmt"

	"code.fbi.h-da.de/danet/ekms/etsiqkdnclient"
	gnmitargetygot "code.fbi.h-da.de/danet/ekms/model"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
)

// AssignForwardingHandler is the implementation of a gnmitarget.PathHandler.
type AssignForwardingHandler struct {
	name       string
	paths      map[string]struct{}
	qkdnClient etsiqkdnclient.EtsiQkdClient
}

func NewAssignForwardingHandler(client etsiqkdnclient.EtsiQkdClient) *AssignForwardingHandler {
	return &AssignForwardingHandler{
		name: "assign-forwarding-handler",
		paths: map[string]struct{}{
			"/assign-forwarding": struct{}{},
		},
		qkdnClient: client,
	}
}

func (yh *AssignForwardingHandler) Name() string {
	return yh.name
}

func (yh *AssignForwardingHandler) Paths() map[string]struct{} {
	return yh.paths
}

func (yh *AssignForwardingHandler) Init(c ygot.ValidatedGoStruct) error {
	// A initialization is not needed for this handler. The default startup has
	// no assign forwarding applied.
	return nil
}

func (yh *AssignForwardingHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	config, ok := c.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("failed type assertion for newConfig %T", (*gnmitargetygot.Gnmitarget)(nil))
	}

	forwarding := config.GetOrCreateAssignForwarding()

	nextHop := forwarding.GetOrCreateNextHop()
	prevHop := forwarding.GetOrCreatePrevHop()

	var nextHopString string
	var prevHopString string

	if nextHop.IpAddress != nil {
		nextHopString = fmt.Sprintf("%s:%d", nextHop.GetIpAddress(), nextHop.GetPort())
	}
	if prevHop.IpAddress != nil {
		prevHopString = fmt.Sprintf("%s:%d", prevHop.GetIpAddress(), prevHop.GetPort())
	}

	if err := yh.qkdnClient.GetEKMS().AssignForwardingRoute(forwarding.GetPathId(), prevHopString, nextHopString); err != nil {
		return err
	}

	return nil
}
