package system

import (
	"fmt"

	gnmitargetygot "code.fbi.h-da.de/danet/ekms/model"
	"code.fbi.h-da.de/danet/ekms/osclient"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
	"github.com/sirupsen/logrus"
)

// HostnameHandler is the implementation of a gnmitarget.PathHandler.
type HostnameHandler struct {
	name     string
	paths    map[string]struct{}
	osClient osclient.Osclient
	weight   int
}

func NewHostnameHandler() *HostnameHandler {
	return &HostnameHandler{
		name: "openconfig-hostname-handler",
		paths: map[string]struct{}{
			"/system/config/hostname": struct{}{},
		},
		osClient: osclient.NewOsClient(),
	}
}

func (yh *HostnameHandler) Name() string {
	return yh.name
}

func (yh *HostnameHandler) Paths() map[string]struct{} {
	return yh.paths
}

func (yh *HostnameHandler) Init(c ygot.ValidatedGoStruct) error {
	config, ok := c.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("failed type assertion for config %T", (*gnmitargetygot.Gnmitarget)(nil))
	}

	confSystem := config.GetOrCreateSystem()
	if err := updateOrCreateHostname(confSystem, yh.osClient); err != nil {
		return err
	}
	return nil
}

func (yh *HostnameHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	fmt.Println("Update request received for ", yh.name)
	config, ok := c.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("failed type assertion for config %T", (*gnmitargetygot.Gnmitarget)(nil))
	}

	system := config.GetSystem()

	if system != nil {
		if config := system.GetOrCreateConfig(); config != nil {
			if hostname := config.GetHostname(); hostname != "" {
				err := yh.osClient.SetHostname(&hostname)
				if err != nil {
					logrus.Debug("Failed to set hostname: ", err)
					return err
				}
			}
		}
	}
	return nil
}

func updateOrCreateHostname(confSystem *gnmitargetygot.OpenconfigSystem_System, os osclient.Osclient) error {
	if config := confSystem.GetOrCreateConfig(); config != nil {
		h, err := os.GetHostname()
		if err != nil {
			return err
		}
		config.Hostname = &h
	}

	//validate struct
	if err := confSystem.Validate(); err != nil {
		return err
	}

	return nil
}
