package system

import (
	"fmt"

	gnmitargetygot "code.fbi.h-da.de/danet/ekms/model"
	"code.fbi.h-da.de/danet/ekms/osclient"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
)

// MemoryHandler is the implementation of a gnmitarget.PathHandler.
type MemoryHandler struct {
	name     string
	paths    map[string]struct{}
	osClient osclient.Osclient
}

func NewMemoryHandler() *MemoryHandler {
	return &MemoryHandler{
		name: "openconfig-memory-handler",
		paths: map[string]struct{}{
			"/system/memory/state": struct{}{},
		},
		osClient: osclient.NewOsClient(),
	}
}

func (yh *MemoryHandler) Name() string {
	return yh.name
}

func (yh *MemoryHandler) Paths() map[string]struct{} {
	return yh.paths
}

func (yh *MemoryHandler) Init(c ygot.ValidatedGoStruct) error {
	config, ok := c.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("failed type assertion for config %T", (*gnmitargetygot.Gnmitarget)(nil))
	}

	confSystem := config.GetOrCreateSystem()
	if err := updateOrCreateMemory(confSystem, yh.osClient); err != nil {
		return err
	}
	return nil
}

func (yh *MemoryHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	fmt.Println("Update request received for ", yh.name)
	return nil
}

func updateOrCreateMemory(confSystem *gnmitargetygot.OpenconfigSystem_System, os osclient.Osclient) error {
	if memory := confSystem.GetOrCreateMemory(); memory != nil {
		memory.GetOrCreateState().Physical = ygot.Uint64(os.GetTotalMemory())
		memory.GetOrCreateState().Free = ygot.Uint64(os.GetFreeMemory())
		memory.GetOrCreateState().Used = ygot.Uint64(os.GetUsedMemory())
	}

	//validate struct
	if err := confSystem.Validate(); err != nil {
		return err
	}

	return nil
}
