package system

import (
	"fmt"

	gnmitargetygot "code.fbi.h-da.de/danet/ekms/model"
	"code.fbi.h-da.de/danet/ekms/osclient"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
	"github.com/sirupsen/logrus"
)

// MotdHandler is the implementation of a gnmitarget.PathHandler.
type MotdHandler struct {
	name     string
	paths    map[string]struct{}
	osClient osclient.Osclient
}

func NewMotdHandler() *MotdHandler {
	return &MotdHandler{
		name: "openconfig-motd-handler",
		paths: map[string]struct{}{
			"/system/config/motd-banner": struct{}{},
		},
		osClient: osclient.NewOsClient(),
	}
}

func (yh *MotdHandler) Name() string {
	return yh.name
}

func (yh *MotdHandler) Paths() map[string]struct{} {
	return yh.paths
}

func (yh *MotdHandler) Init(c ygot.ValidatedGoStruct) error {
	config, ok := c.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("failed type assertion for config %T", (*gnmitargetygot.Gnmitarget)(nil))
	}

	confSystem := config.GetOrCreateSystem()
	if err := updateOrCreateMotd(confSystem, yh.osClient); err != nil {
		return err
	}
	return nil
}

func (yh *MotdHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	fmt.Println("Update request received for ", yh.name)
	config, ok := c.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("failed type assertion for config %T", (*gnmitargetygot.Gnmitarget)(nil))
	}

	system := config.GetSystem()

	if system != nil {
		if config := system.GetOrCreateConfig(); config != nil {
			if motd := config.GetMotdBanner(); motd != "" {
				err := yh.osClient.SetMotd(motd)
				if err != nil {
					logrus.Debug("Failed to set motd banner: ", err)
					return err
				}
			}
		}
	}
	return nil
}

func updateOrCreateMotd(confSystem *gnmitargetygot.OpenconfigSystem_System, os osclient.Osclient) error {
	if config := confSystem.GetOrCreateConfig(); config != nil {
		motd, err := os.GetMotd()
		if err != nil {
			return err
		}
		config.MotdBanner = ygot.String(motd)
	}

	//validate struct
	if err := confSystem.Validate(); err != nil {
		return err
	}

	return nil
}
