package system

import (
	"fmt"
	"time"

	gnmitargetygot "code.fbi.h-da.de/danet/ekms/model"
	"code.fbi.h-da.de/danet/ekms/osclient"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
	gopshost "github.com/shirou/gopsutil/host"
)

// StateHandler is the implementation of a gnmitarget.PathHandler.
type StateHandler struct {
	name     string
	paths    map[string]struct{}
	osClient osclient.Osclient
}

func NewStateHandler() *StateHandler {
	return &StateHandler{
		name: "openconfig-memory-handler",
		paths: map[string]struct{}{
			"/system/state": struct{}{},
		},
		osClient: osclient.NewOsClient(),
	}
}

func (yh *StateHandler) Name() string {
	return yh.name
}

func (yh *StateHandler) Paths() map[string]struct{} {
	return yh.paths
}

func (yh *StateHandler) Init(c ygot.ValidatedGoStruct) error {
	config, ok := c.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("failed type assertion for config %T", (*gnmitargetygot.Gnmitarget)(nil))
	}

	confSystem := config.GetOrCreateSystem()
	if err := updateOrCreateState(confSystem, yh.osClient); err != nil {
		return err
	}
	return nil
}

func (yh *StateHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	fmt.Println("Update request received for ", yh.name)
	return nil
}

func updateOrCreateState(confSystem *gnmitargetygot.OpenconfigSystem_System, os osclient.Osclient) error {
	if state := confSystem.GetOrCreateState(); state != nil {
		state.CurrentDatetime = ygot.String(time.Now().Format(time.RFC3339))
		bootTime, err := gopshost.BootTime()
		if err != nil {
			return err
		}
		state.BootTime = ygot.Uint64(bootTime)

		// Read version of the operating system
		// TODO: better error handling, or even better error handling at all
		osVersion, _ := os.GetSoftwareVersion()
		state.SoftwareVersion = ygot.String(osVersion)
	}

	//validate struct
	if err := confSystem.Validate(); err != nil {
		return err
	}

	return nil
}
