package system

import (
	"fmt"
	"strings"

	gnmitargetygot "code.fbi.h-da.de/danet/ekms/model"
	"code.fbi.h-da.de/danet/ekms/osclient"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
)

// SystemHandler is the implementation of a gnmitarget.PathHandler.
type SystemHandler struct {
	name     string
	paths    map[string]struct{}
	osClient osclient.Osclient
}

func NewSystemHandler() *SystemHandler {
	return &SystemHandler{
		name: "openconfig-system-handler",
		paths: map[string]struct{}{
			"/system": struct{}{},
		},
		osClient: osclient.NewOsClient(),
	}
}

func (yh *SystemHandler) Name() string {
	return yh.name
}

func (yh *SystemHandler) Paths() map[string]struct{} {
	return yh.paths
}

func (yh *SystemHandler) Init(c ygot.ValidatedGoStruct) error {
	config, ok := c.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("failed type assertion for config %T", (*gnmitargetygot.Gnmitarget)(nil))
	}

	confSystem := config.GetOrCreateSystem()
	if err := updateOrCreateSystem(confSystem, yh.osClient); err != nil {
		return err
	}
	return nil
}

func (yh *SystemHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	fmt.Println("Update request received for ", yh.name)
	return nil
}

func updateOrCreateSystem(confSystem *gnmitargetygot.OpenconfigSystem_System, os osclient.Osclient) error {
	if config := confSystem.GetOrCreateConfig(); config != nil {
		domain, err := os.GetDomainName()
		if err != nil {
			return err
		}
		config.DomainName = ygot.String(domain)
	}

	if clock := confSystem.GetOrCreateClock(); clock != nil {
		if config := clock.GetOrCreateConfig(); config != nil {
			config.TimezoneName = ygot.String(strings.TrimSuffix(os.GetTimeZoneName(), "\n"))
		}
	}

	//validate struct
	if err := confSystem.Validate(); err != nil {
		return err
	}

	return nil
}
